@@ -127,7 +127,12 @@ class BlockStore:
         return None
 
     def rollback_cache_block(self, header_hash: bytes32):
-        self.block_cache.remove(header_hash)
+        try:
+            self.block_cache.remove(header_hash)
+        except KeyError:
+            # this is best effort. When rolling back, we may not have added the
+            # block to the cache yet
+            pass
 
     async def get_full_block(self, header_hash: bytes32) -> Optional[FullBlock]:
         cached = self.block_cache.get(header_hash)
@@ -232,26 +237,6 @@ class BlockStore:
             return BlockRecord.from_bytes(row[0])
         return None
 
-    async def get_block_records(
-        self,
-    ) -> Tuple[Dict[bytes32, BlockRecord], Optional[bytes32]]:
-        """
-        Returns a dictionary with all blocks, as well as the header hash of the peak,
-        if present.
-        """
-        cursor = await self.db.execute("SELECT * from block_records")
-        rows = await cursor.fetchall()
-        await cursor.close()
-        ret: Dict[bytes32, BlockRecord] = {}
-        peak: Optional[bytes32] = None
-        for row in rows:
-            header_hash = bytes.fromhex(row[0])
-            ret[header_hash] = BlockRecord.from_bytes(row[3])
-            if row[5]:
-                assert peak is None  # Sanity check, only one peak
-                peak = header_hash
-        return ret, peak
-
     async def get_block_records_in_range(
         self,
         start: int,
