@@ -100,7 +100,9 @@ class FullNodeAPI:
         A peer notifies us that they have added a new peak to their blockchain. If we don't have it,
         we can ask for it.
         """
-        async with self.full_node.new_peak_lock:
+        # this semaphore limits the number of tasks that can call new_peak() at
+        # the same time, since it can be expensive
+        async with self.full_node.new_peak_sem:
             return await self.full_node.new_peak(request, peer)
 
     @peer_required
@@ -1277,7 +1279,9 @@ class FullNodeAPI:
     async def new_compact_vdf(self, request: full_node_protocol.NewCompactVDF, peer: ws.WSChiaConnection):
         if self.full_node.sync_store.get_sync_mode():
             return None
-        async with self.full_node.compact_vdf_lock:
+        # this semaphore will only allow a limited number of tasks call
+        # new_compact_vdf() at a time, since it can be expensive
+        async with self.full_node.compact_vdf_sem:
             await self.full_node.new_compact_vdf(request, peer)
 
     @peer_required
