@@ -3,9 +3,13 @@ from typing import Any, Callable, Dict, List, Optional
 from chia.consensus.block_record import BlockRecord
 from chia.consensus.pos_quality import UI_ACTUAL_SPACE_CONSTANT_FACTOR
 from chia.full_node.full_node import FullNode
+from chia.full_node.mempool_check_conditions import get_puzzle_and_solution_for_coin
+from chia.types.blockchain_format.program import Program, SerializedProgram
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.coin_record import CoinRecord
+from chia.types.coin_solution import CoinSolution
 from chia.types.full_block import FullBlock
+from chia.types.generator_types import BlockGenerator
 from chia.types.mempool_inclusion_status import MempoolInclusionStatus
 from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_header_block import UnfinishedHeaderBlock
@@ -34,11 +38,13 @@ class FullNodeRpcApi:
             "/get_additions_and_removals": self.get_additions_and_removals,
             "/get_initial_freeze_period": self.get_initial_freeze_period,
             "/get_network_info": self.get_network_info,
+            "/get_recent_signage_point_or_eos": self.get_recent_signage_point_or_eos,
             # Coins
             "/get_coin_records_by_puzzle_hash": self.get_coin_records_by_puzzle_hash,
             "/get_coin_records_by_puzzle_hashes": self.get_coin_records_by_puzzle_hashes,
             "/get_coin_record_by_name": self.get_coin_record_by_name,
             "/push_tx": self.push_tx,
+            "/get_puzzle_and_solution": self.get_puzzle_and_solution,
             # Mempool
             "/get_all_mempool_tx_ids": self.get_all_mempool_tx_ids,
             "/get_all_mempool_items": self.get_all_mempool_items,
@@ -105,6 +111,10 @@ class FullNodeRpcApi:
                 assert sync_tip_height is not None
             if peak is not None:
                 sync_progress_height: uint32 = peak.height
+                # Don't display we're syncing towards 0, instead show 'Syncing height/height'
+                # until sync_store retrieves the correct number.
+                if sync_tip_height == uint32(0):
+                    sync_tip_height = peak.height
             else:
                 sync_progress_height = uint32(0)
         else:
@@ -157,6 +167,95 @@ class FullNodeRpcApi:
         address_prefix = self.service.config["network_overrides"]["config"][network_name]["address_prefix"]
         return {"network_name": network_name, "network_prefix": address_prefix}
 
+    async def get_recent_signage_point_or_eos(self, request: Dict):
+        if "sp_hash" not in request:
+            challenge_hash: bytes32 = hexstr_to_bytes(request["challenge_hash"])
+            # This is the case of getting an end of slot
+            eos_tuple = self.service.full_node_store.recent_eos.get(challenge_hash)
+            if not eos_tuple:
+                raise ValueError(f"Did not find eos {challenge_hash.hex()} in cache")
+            eos, time_received = eos_tuple
+
+            # If it's still in the full node store, it's not reverted
+            if self.service.full_node_store.get_sub_slot(eos.challenge_chain.get_hash()):
+                return {"eos": eos, "time_received": time_received, "reverted": False}
+
+            # Otherwise we can backtrack from peak to find it in the blockchain
+            curr: Optional[BlockRecord] = self.service.blockchain.get_peak()
+            if curr is None:
+                raise ValueError("No blocks in the chain")
+
+            number_of_slots_searched = 0
+            while number_of_slots_searched < 10:
+                if curr.first_in_sub_slot:
+                    assert curr.finished_challenge_slot_hashes is not None
+                    if curr.finished_challenge_slot_hashes[-1] == eos.challenge_chain.get_hash():
+                        # Found this slot in the blockchain
+                        return {"eos": eos, "time_received": time_received, "reverted": False}
+                    number_of_slots_searched += len(curr.finished_challenge_slot_hashes)
+                curr = self.service.blockchain.try_block_record(curr.prev_hash)
+                if curr is None:
+                    # Got to the beginning of the blockchain without finding the slot
+                    return {"eos": eos, "time_received": time_received, "reverted": True}
+
+            # Backtracked through 10 slots but still did not find it
+            return {"eos": eos, "time_received": time_received, "reverted": True}
+
+        # Now we handle the case of getting a signage point
+        sp_hash: bytes32 = hexstr_to_bytes(request["sp_hash"])
+        sp_tuple = self.service.full_node_store.recent_signage_points.get(sp_hash)
+        if sp_tuple is None:
+            raise ValueError(f"Did not find sp {sp_hash.hex()} in cache")
+
+        sp, time_received = sp_tuple
+
+        # If it's still in the full node store, it's not reverted
+        if self.service.full_node_store.get_signage_point(sp_hash):
+            return {"signage_point": sp, "time_received": time_received, "reverted": False}
+
+        # Otherwise we can backtrack from peak to find it in the blockchain
+        rc_challenge: bytes32 = sp.rc_vdf.challenge
+        next_b: Optional[BlockRecord] = None
+        curr_b_optional: Optional[BlockRecord] = self.service.blockchain.get_peak()
+        assert curr_b_optional is not None
+        curr_b: BlockRecord = curr_b_optional
+
+        for _ in range(200):
+            sp_total_iters = sp.cc_vdf.number_of_iterations + curr_b.ip_sub_slot_total_iters(self.service.constants)
+            if curr_b.reward_infusion_new_challenge == rc_challenge:
+                if next_b is None:
+                    return {"signage_point": sp, "time_received": time_received, "reverted": False}
+                next_b_total_iters = next_b.ip_sub_slot_total_iters(self.service.constants) + next_b.ip_iters(
+                    self.service.constants
+                )
+
+                return {
+                    "signage_point": sp,
+                    "time_received": time_received,
+                    "reverted": sp_total_iters > next_b_total_iters,
+                }
+            if curr_b.finished_reward_slot_hashes is not None:
+                assert curr_b.finished_challenge_slot_hashes is not None
+                for eos_rc in curr_b.finished_challenge_slot_hashes:
+                    if eos_rc == rc_challenge:
+                        if next_b is None:
+                            return {"signage_point": sp, "time_received": time_received, "reverted": False}
+                        next_b_total_iters = next_b.ip_sub_slot_total_iters(self.service.constants) + next_b.ip_iters(
+                            self.service.constants
+                        )
+                        return {
+                            "signage_point": sp,
+                            "time_received": time_received,
+                            "reverted": sp_total_iters > next_b_total_iters,
+                        }
+            next_b = curr_b
+            curr_b_optional = self.service.blockchain.try_block_record(curr_b.prev_hash)
+            if curr_b_optional is None:
+                break
+            curr_b = curr_b_optional
+
+        return {"signage_point": sp, "time_received": time_received, "reverted": True}
+
     async def get_block(self, request: Dict) -> Optional[Dict]:
         if "header_hash" not in request:
             raise ValueError("No header_hash in request")
@@ -389,6 +488,31 @@ class FullNodeRpcApi:
             "status": status.name,
         }
 
+    async def get_puzzle_and_solution(self, request: Dict) -> Optional[Dict]:
+        coin_name: bytes32 = hexstr_to_bytes(request["coin_id"])
+        height = request["height"]
+        coin_record = await self.service.coin_store.get_coin_record(coin_name)
+        if coin_record is None or not coin_record.spent or coin_record.spent_block_index != height:
+            raise ValueError(f"Invalid height {height}. coin record {coin_record}")
+
+        header_hash = self.service.blockchain.height_to_hash(height)
+        block: Optional[FullBlock] = await self.service.block_store.get_full_block(header_hash)
+
+        if block is None or block.transactions_generator is None:
+            raise ValueError("Invalid block or block generator")
+
+        block_generator: Optional[BlockGenerator] = await self.service.blockchain.get_block_generator(block)
+        assert block_generator is not None
+        error, puzzle, solution = get_puzzle_and_solution_for_coin(
+            block_generator, coin_name, self.service.constants.MAX_BLOCK_COST_CLVM
+        )
+        if error is not None:
+            raise ValueError(f"Error: {error}")
+
+        puzzle_ser: SerializedProgram = SerializedProgram.from_program(Program.to(puzzle))
+        solution_ser: SerializedProgram = SerializedProgram.from_program(Program.to(solution))
+        return {"coin_solution": CoinSolution(coin_record.coin, puzzle_ser, solution_ser)}
+
     async def get_additions_and_removals(self, request: Dict) -> Optional[Dict]:
         if "header_hash" not in request:
             raise ValueError("No header_hash in request")
