@@ -1,9 +1,12 @@
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Optional, Tuple, Any
 
 from chia.consensus.block_record import BlockRecord
+from chia.full_node.signage_point import SignagePoint
 from chia.rpc.rpc_client import RpcClient
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.coin_record import CoinRecord
+from chia.types.coin_solution import CoinSolution
+from chia.types.end_of_slot_bundle import EndOfSubSlotBundle
 from chia.types.full_block import FullBlock
 from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_header_block import UnfinishedHeaderBlock
@@ -72,6 +75,13 @@ class FullNodeRpcClient(RpcClient):
             return None
         return network_space_bytes_estimate["space"]
 
+    async def get_coin_record_by_name(self, coin_id: bytes32) -> Optional[CoinRecord]:
+        try:
+            response = await self.fetch("get_coin_record_by_name", {"name": coin_id.hex()})
+        except Exception:
+            return None
+        return CoinRecord.from_json_dict(response["coin_record"])
+
     async def get_coin_records_by_puzzle_hash(
         self,
         puzzle_hash: bytes32,
@@ -86,7 +96,7 @@ class FullNodeRpcClient(RpcClient):
             d["end_height"] = end_height
         return [
             CoinRecord.from_json_dict(coin)
-            for coin in ((await self.fetch("get_coin_records_by_puzzle_hash", d))["coin_records"])
+            for coin in (await self.fetch("get_coin_records_by_puzzle_hash", d))["coin_records"]
         ]
 
     async def get_coin_records_by_puzzle_hashes(
@@ -104,7 +114,7 @@ class FullNodeRpcClient(RpcClient):
             d["end_height"] = end_height
         return [
             CoinRecord.from_json_dict(coin)
-            for coin in ((await self.fetch("get_coin_records_by_puzzle_hashes", d))["coin_records"])
+            for coin in (await self.fetch("get_coin_records_by_puzzle_hashes", d))["coin_records"]
         ]
 
     async def get_additions_and_removals(self, header_hash: bytes32) -> Tuple[List[CoinRecord], List[CoinRecord]]:
@@ -133,6 +143,13 @@ class FullNodeRpcClient(RpcClient):
     async def push_tx(self, spend_bundle: SpendBundle):
         return await self.fetch("push_tx", {"spend_bundle": spend_bundle.to_json_dict()})
 
+    async def get_puzzle_and_solution(self, coin_id: bytes32, height: uint32) -> Optional[CoinRecord]:
+        try:
+            response = await self.fetch("get_puzzle_and_solution", {"coin_id": coin_id.hex(), "height": height})
+            return CoinSolution.from_json_dict(response["coin_solution"])
+        except Exception:
+            return None
+
     async def get_all_mempool_tx_ids(self) -> List[bytes32]:
         response = await self.fetch("get_all_mempool_tx_ids", {})
         return [bytes32(hexstr_to_bytes(tx_id_hex)) for tx_id_hex in response["tx_ids"]]
@@ -150,3 +167,26 @@ class FullNodeRpcClient(RpcClient):
             return response["mempool_item"]
         except Exception:
             return None
+
+    async def get_recent_signage_point_or_eos(
+        self, sp_hash: Optional[bytes32], challenge_hash: Optional[bytes32]
+    ) -> Optional[Any]:
+        try:
+            if sp_hash is not None:
+                assert challenge_hash is None
+                response = await self.fetch("get_recent_signage_point_or_eos", {"sp_hash": sp_hash.hex()})
+                return {
+                    "signage_point": SignagePoint.from_json_dict(response["signage_point"]),
+                    "time_received": response["time_received"],
+                    "reverted": response["reverted"],
+                }
+            else:
+                assert challenge_hash is not None
+                response = await self.fetch("get_recent_signage_point_or_eos", {"challenge_hash": challenge_hash.hex()})
+                return {
+                    "eos": EndOfSubSlotBundle.from_json_dict(response["eos"]),
+                    "time_received": response["time_received"],
+                    "reverted": response["reverted"],
+                }
+        except Exception:
+            return None
