@@ -181,6 +181,9 @@ class Timelord:
         if found_index == -1:
             log.warning(f"Will not infuse {block.rc_prev} because its reward chain challenge is not in the chain")
             return None
+        if ip_iters > block_ip_iters:
+            log.warning("Too late to infuse block")
+            return None
 
         new_block_iters = uint64(block_ip_iters - ip_iters)
         if len(self.last_state.reward_challenge_cache) > found_index + 1:
