@@ -1,7 +1,7 @@
 from typing import List
 
 from chia.types.blockchain_format.coin import Coin
-from chia.types.blockchain_format.program import Program
+from chia.types.blockchain_format.program import SerializedProgram
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.condition_tools import (
     conditions_dict_for_solution,
@@ -9,7 +9,9 @@ from chia.util.condition_tools import (
 )
 
 
-def additions_for_solution(coin_name: bytes32, puzzle_reveal: Program, solution: Program, max_cost: int) -> List[Coin]:
+def additions_for_solution(
+    coin_name: bytes32, puzzle_reveal: SerializedProgram, solution: SerializedProgram, max_cost: int
+) -> List[Coin]:
     """
     Checks the conditions created by CoinSolution and returns the list of all coins created
     """
