@@ -1,3 +1,4 @@
+from dataclasses import dataclass
 from typing import Dict, List, Optional, Tuple
 
 import aiosqlite
@@ -5,13 +6,21 @@ import aiosqlite
 from chia.consensus.block_record import BlockRecord
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.blockchain_format.sub_epoch_summary import SubEpochSummary
+from chia.types.coin_solution import CoinSolution
 from chia.types.header_block import HeaderBlock
 from chia.util.db_wrapper import DBWrapper
 from chia.util.ints import uint32, uint64
 from chia.util.lru_cache import LRUCache
+from chia.util.streamable import Streamable, streamable
 from chia.wallet.block_record import HeaderBlockRecord
 
 
+@dataclass(frozen=True)
+@streamable
+class AdditionalCoinSpends(Streamable):
+    coin_spends_list: List[CoinSolution]
+
+
 class WalletBlockStore:
     """
     This object handles HeaderBlocks and Blocks stored in DB used by wallet.
@@ -48,6 +57,10 @@ class WalletBlockStore:
             "block blob, sub_epoch_summary blob, is_peak tinyint)"
         )
 
+        await self.db.execute(
+            "CREATE TABLE IF NOT EXISTS additional_coin_spends(header_hash text PRIMARY KEY, spends_list_blob blob)"
+        )
+
         # Height index so we can look up in order of height for sync purposes
         await self.db.execute("CREATE INDEX IF NOT EXISTS height on block_records(height)")
 
@@ -62,7 +75,12 @@ class WalletBlockStore:
         await cursor_2.close()
         await self.db.commit()
 
-    async def add_block_record(self, header_block_record: HeaderBlockRecord, block_record: BlockRecord):
+    async def add_block_record(
+        self,
+        header_block_record: HeaderBlockRecord,
+        block_record: BlockRecord,
+        additional_coin_spends: List[CoinSolution],
+    ):
         """
         Adds a block record to the database. This block record is assumed to be connected
         to the chain, but it may or may not be in the LCA path.
@@ -103,9 +121,16 @@ class WalletBlockStore:
                 False,
             ),
         )
-
         await cursor_2.close()
 
+        if len(additional_coin_spends) > 0:
+            blob: bytes = bytes(AdditionalCoinSpends(additional_coin_spends))
+            cursor_3 = await self.db.execute(
+                "INSERT OR REPLACE INTO additional_coin_spends VALUES(?, ?)",
+                (header_block_record.header_hash.hex(), blob),
+            )
+            await cursor_3.close()
+
     async def get_header_block_at(self, heights: List[uint32]) -> List[HeaderBlock]:
         if len(heights) == 0:
             return []
@@ -132,6 +157,18 @@ class WalletBlockStore:
         else:
             return None
 
+    async def get_additional_coin_spends(self, header_hash: bytes32) -> Optional[List[CoinSolution]]:
+        cursor = await self.db.execute(
+            "SELECT spends_list_blob from additional_coin_spends WHERE header_hash=?", (header_hash.hex(),)
+        )
+        row = await cursor.fetchone()
+        await cursor.close()
+        if row is not None:
+            coin_spends: AdditionalCoinSpends = AdditionalCoinSpends.from_bytes(row[0])
+            return coin_spends.coin_spends_list
+        else:
+            return None
+
     async def get_block_record(self, header_hash: bytes32) -> Optional[BlockRecord]:
         cursor = await self.db.execute(
             "SELECT block from block_records WHERE header_hash=?",
@@ -164,6 +201,9 @@ class WalletBlockStore:
                 peak = header_hash
         return ret, peak
 
+    def rollback_cache_block(self, header_hash: bytes32):
+        self.block_cache.remove(header_hash)
+
     async def set_peak(self, header_hash: bytes32) -> None:
         cursor_1 = await self.db.execute("UPDATE block_records SET is_peak=0 WHERE is_peak=1")
         await cursor_1.close()
