@@ -1,5 +1,6 @@
 # flake8: noqa: F811, F401
 import asyncio
+import logging
 from secrets import token_bytes
 from typing import List, Optional
 
@@ -15,11 +16,14 @@ from chia.protocols.timelord_protocol import NewInfusionPointVDF
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.unfinished_block import UnfinishedBlock
 from chia.util.block_cache import BlockCache
-from chia.util.block_tools import get_signage_point
+from tests.block_tools import get_signage_point, BlockTools
 from chia.util.hash import std_hash
 from chia.util.ints import uint8, uint32, uint64, uint128
-from tests.core.fixtures import default_1000_blocks, empty_blockchain  # noqa: F401
-from tests.setup_nodes import bt, test_constants
+from tests.core.fixtures import default_1000_blocks, create_blockchain  # noqa: F401
+from tests.setup_nodes import test_constants as test_constants_original
+
+test_constants = test_constants_original.replace(**{"DISCRIMINANT_SIZE_BITS": 32, "SUB_SLOT_ITERS_STARTING": 2 ** 12})
+bt = BlockTools(test_constants)
 
 
 @pytest.fixture(scope="session")
@@ -28,6 +32,27 @@ def event_loop():
     yield loop
 
 
+log = logging.getLogger(__name__)
+
+
+@pytest.fixture(scope="function")
+async def empty_blockchain():
+    bc1, connection, db_path = await create_blockchain(test_constants)
+    yield bc1
+    await connection.close()
+    bc1.shut_down()
+    db_path.unlink()
+
+
+@pytest.fixture(scope="function")
+async def empty_blockchain_original():
+    bc1, connection, db_path = await create_blockchain(test_constants_original)
+    yield bc1
+    await connection.close()
+    bc1.shut_down()
+    db_path.unlink()
+
+
 class TestFullNodeStore:
     @pytest.mark.asyncio
     async def test_basic_store(self, empty_blockchain, normalized_to_identity: bool = False):
@@ -576,10 +601,13 @@ class TestFullNodeStore:
         # SP, B2 SP .... SP B1
         #     i2 .........  i1
         # Then do a reorg up to B2, removing all signage points after B2, but not before
+        log.warning(f"Adding blocks up to {blocks[-1]}")
         for block in blocks:
             await blockchain.receive_block(block)
 
+        log.warning(f"Starting loop")
         while True:
+            log.warning("Looping")
             blocks = bt.get_consecutive_blocks(1, block_list_input=blocks, skip_slots=1)
             assert (await blockchain.receive_block(blocks[-1]))[0] == ReceiveBlockResult.NEW_PEAK
             peak = blockchain.get_peak()
@@ -689,16 +717,16 @@ class TestFullNodeStore:
         await self.test_basic_store(empty_blockchain, True)
 
     @pytest.mark.asyncio
-    async def test_long_chain_slots(self, empty_blockchain, default_1000_blocks):
-        blockchain = empty_blockchain
-        store = FullNodeStore(test_constants)
+    async def test_long_chain_slots(self, empty_blockchain_original, default_1000_blocks):
+        blockchain = empty_blockchain_original
+        store = FullNodeStore(test_constants_original)
         blocks = default_1000_blocks
         peak = None
         peak_full_block = None
         for block in blocks:
             for sub_slot in block.finished_sub_slots:
                 assert store.new_finished_sub_slot(sub_slot, blockchain, peak, peak_full_block) is not None
-            res, _, _ = await blockchain.receive_block(block)
+            res, err, _ = await blockchain.receive_block(block)
             assert res == ReceiveBlockResult.NEW_PEAK
             peak = blockchain.get_peak()
             peak_full_block = await blockchain.get_full_peak()
